package	jp.co.sds_corp.ratemixer.fx.mixer;

import java.math.BigDecimal;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.fxutil.ForexCurve;
import jp.co.sds_corp.fxutil.ForexRate;
import jp.co.sds_corp.fxutil.IRateLoader;
import jp.co.sds_corp.fxutil.PricingException;
import jp.co.sds_corp.ratemixer.fx.FxRate;

public class RateHolder implements IRateChangeListener
{
	IRateLoader	loader;
	RateManager	manager;
	CcyPair		pair;
	CcyPair[]	cross;
	ForexCurve curve;

	public RateHolder( RateManager manager, CcyPair pair ) {
		this.manager = manager;
		loader = manager.getLoader();
		this.pair = pair;
		curve = new ForexCurve( pair );

		if ( pair.isCompound()) {
			cross = curve.getCross();
			for ( int i = 0; i < cross.length; i++ ) {
				manager.addRateChangeListener( cross[i], this );
			}
			curve.setRateLoader( loader );
		} else {
			manager.addRateChangeListener( pair, this );
		}
	}

	public void rateChanged( FxRate rate ) {
		if ( pair.isCompound()) {
			calcCross( rate );
		} else {
			saveRate( rate );
		}
	}

	private void calcCross( FxRate rate ) {
		// @@@
		if ( !rate.getTerm().equals( "SP" )) {
			return;
		}
		// @@@ クロスの算出
	}

	private void saveRate( FxRate rate ) {
		try {
			ForexRate fxr = curve.getTerm( rate.getTerm() );
			if ( fxr == null ) {
				fxr = curve.addTerm( rate.getTerm() );
			}
			rate.copyTo( fxr );
			fireRateChange( rate );
		} catch ( PricingException ex ) {
		}
	}

	public ForexCurve getForexCurve() {
		return curve;
	}

	protected void fireRateChange( FxRate rate ) {
		if ( rate.getTerm().equals( "SP" )) {
			adjust( rate );
		}
		// @@@ 現レートとの比較
		manager.fireRateChange( rate );
	}

	protected void adjust( FxRate rate ) {
		int bias = 0;
		// @@@ spread skew
		String var;
		var = rate.getProperty( "Bias" );
		if ( var != null && var.equals( "1" )) {
			bias = 1;
		}
		var = rate.getProperty( "Spread" );
		if ( var != null ) {
			rate.setProperty( "OrgBid", String.valueOf( rate.getBid()));
			rate.setProperty( "OrgAsk", String.valueOf( rate.getAsk()));
			BigDecimal spread = new BigDecimal( var )
				.movePointLeft( pair.getPoint() );
			BigDecimal bid = rate.getAsk().add( rate.getBid() )
				.subtract( spread );
			BigDecimal bid1 = 
				bid.divide( new BigDecimal( "2" ), pair.getPoint() + 1,
					BigDecimal.ROUND_DOWN );
			BigDecimal bid2 =
				bid.divide( new BigDecimal( "2" ), pair.getPoint(),
					BigDecimal.ROUND_DOWN );
			if ( bid1.compareTo( bid2 ) == 0 ) {
				rate.setBid( bid2 );
			} else {
				if ( bias == 0 ) {
					rate.setBid( bid2 );
				} else {
					bid = bid.divide( new BigDecimal( "2" ), pair.getPoint(),
						BigDecimal.ROUND_UP );
					rate.setBid( bid );
				}
			}
			BigDecimal ask = rate.getBid().add( spread );
			rate.setAsk( ask );
		}
		var = rate.getProperty( "Skew" );
		if ( var != null ) {
			BigDecimal skew = new BigDecimal( var )
				.movePointLeft( pair.getPoint() );
			rate.setBid( rate.getBid().add( skew ));
			rate.setAsk( rate.getAsk().add( skew ));
		}
	}

}
