package	jp.co.sds_corp.ratemixer.fx.source;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.fxutil.MasterCache;
import jp.co.sds_corp.fxutil.TermList;
import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.transfer.IData;
import jp.co.sds_corp.transfer.source.AbstractSource;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* レート取得元オブジェクトの基底クラスです。
*/
public abstract class FxSource extends AbstractSource
{
	private static final long serialVersionUID = 1L;
	HashMap<String,FxRate>		mapRate = new HashMap<String,FxRate>();
	int			nRates;
	MasterCache	master;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		nRates = 0;
		master = MasterCache.getInstance();
		Map<String,CcyPair> map = master.mapCcyPair();
		Iterator<CcyPair> itr = map.values().iterator();
		while ( itr.hasNext() ) {
			CcyPair pair = (CcyPair )itr.next();
			if ( !pair.isFeed() ) {
				continue;
			}

			String[] terms = TermList.getDefaultTerms();
			for ( int i = 0; i < terms.length; i++ ) {
				String term = terms[i];
				FxRate rate;
				rate = new FxRate( pair, term );
				rate.setInit( true );
				String pkey = pair.getCcy1().getCode()+pair.getCcy2().getCode()
								+ term;
				getRateMap().put( pkey, rate );
			}
			nRates++;
		}
	}

	/**
	* 初期化時に取得されたレート情報を取得します。
	* @return	レート情報
	*/
	public synchronized Map<String,FxRate> getRateMap() {
		return mapRate;
	}

	public FxRate getCurrentRate( FxRate rate ) {
		CcyPair pair = rate.getCcyPair();
		String pkey = pair.getCcy1().getCode()+pair.getCcy2().getCode()
								+ rate.getTerm();
		FxRate ret = (FxRate )mapRate.get( pkey );
		return ret;
	}

	public void fireReceive( IData data ) throws Exception {
		super.fireReceive( data );

		FxRate rate = (FxRate )data;
		CcyPair pair = rate.getCcyPair();
		String pkey = pair.getCcy1().getCode()+pair.getCcy2().getCode()
								+ rate.getTerm();
		mapRate.put( pkey, rate );
	}

}
